import enum


__all__ = ('OpCode',)


class OpCode(enum.IntEnum):

    """
    All possible send/receive opcodes.
    """

    identify            = 0
    select_protocol     = 1
    ready               = 2
    heartbeat           = 3
    session_description = 4
    speaking            = 5
    heartbeat_ack       = 6
    resume              = 7
    hello               = 8
    resumed             = 9
    connect             = 18
    disconnect          = 13



import typing

from . import model as _model
from . import voice     as _voice


__all__ = (
    'Ready', 'UpdateApplicationCommandPermission', 'CreateAutoModerationRule', 
    'UpdateAutoModerationRule', 'DeleteAutoModerationRule', 
    'ExecuteAutoModerationRule', 'CreateChannel', 'UpdateChannel', 
    'DeleteChannel', 'CreateThread', 'UpdateThread', 'DeleteThread', 
    'SyncThreads', 'UpdateThreadMember', 'UpdateThreadMembers', 
    'UpdateChannelPins', 'CreateGuild', 'AvailableGuild', 'DeleteGuild', 
    'UnavailableGuild', 'UpdateGuild', 'CreateGuildAuditLogEntry', 
    'CreateGuildBan', 'DeleteGuildBan', 'UpdateGuildEmojis', 
    'UpdateGuildStickers', 'UpdateGuildIntegrations', 'CreateGuildMember', 
    'DeleteGuildMember', 'UpdateGuildMember', 'ReceiveGuildMembers', 
    'CreateGuildRole', 'UpdateGuildRole', 'DeleteGuildRole', 
    'CreateGuildScheduledEvent', 'UpdateGuildScheduledEvent', 
    'DeleteGuildScheduledEvent', 'CreateGuildScheduledEventUser', 
    'DeleteGuildScheduledEventUser', 'CreateIntegration', 'UpdateIntegration', 
    'DeleteIntegration', 'CreateInvite', 'DeleteInvite', 'CreateMessage', 
    'UpdateMessage', 'DeleteMessage', 'DeleteMessages',
    'CreateMessageReaction', 'DeleteMessageReaction', 
    'DeleteAllMessageReactions', 'DeleteAllMessageEmojiReactions', 
    'UpdatePresence', 'CreateTypingIndicator', 'UpdateSelfUser', 
    'UpdateVoiceState', 'UpdateVoiceServer', 'UpdateWebhooks', 
    'CreateInteraction', 'CreateStageInstance', 'UpdateStageInstance', 
    'DeleteStageInstance', 'EnterVoice', 'LeaveVoice', 'Speak'
)


class Ready(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.ready`.
    """


class UpdateApplicationCommandPermission(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.update_application_command_permissions`.
    """

    guild                          : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    application_command_permissions: _model.objects.GuildApplicationCommandPermissions
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          -
    """


class CreateAutoModerationRule(typing.NamedTuple):

    """
    Dispatched on :attr:`.enums.GatewayEvent.create_auto_moderation_rule`.
    """

    guild               : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    auto_moderation_rule: _model.objects.AutoModerationRule
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          - 
    """

class UpdateAutoModerationRule(typing.NamedTuple):

    """
    Dispatched on :attr:`.enums.GatewayEvent.update_auto_moderation_rule`.
    """

    guild               : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    auto_moderation_rule: _model.objects.AutoModerationRule
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          - 
    """
  

class DeleteAutoModerationRule(typing.NamedTuple):

    """
    Dispatched on :attr:`.enums.GatewayEvent.delete_auto_moderation_rule`.
    """

    guild               : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    auto_moderation_rule: _model.objects.AutoModerationRule
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          - 
    """
  

class ExecuteAutoModerationRule(typing.NamedTuple):

    """
    Dispatched on :attr:`.enums.GatewayEvent.execute_auto_moderation_action`.
    """

    guild                 : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    auto_moderation_action: _model.objects.AutoModerationAction
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`action`
          - 
    """
    auto_moderation_rule  : _model.objects.AutoModerationRule
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`rule_id`
          - :attr:`~.model.objects.AutoModerationRule.id`
        * - :code:`rule_trigger_type`
          - :attr:`~.model.objects.AutoModerationRule.trigger_type`
    """
    user                  : _model.objects.User
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`user_id`
          - :attr:`~.model.objects.User.id`
    """
    channel               : _model.objects.Channel
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`channel_id`
          - :attr:`~.model.objects.Channel.id`
    """
    source_message        : _model.objects.Message
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`message_id`
          - :attr:`~.model.objects.Message.id`
        * - :code:`content`
          - :attr:`~.model.objects.Message.content`
    """
    system_message        : _model.objects.Message
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`alert_system_message_id`
          - :attr:`~.model.objects.Message.id`
    """
    matched_keyword       : _model.types.String
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`matched_keyword`
          - 
    """
    matched_content       : _model.types.String
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`matched_content`
          - 
    """
  

class CreateChannel(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.create_channel`.
    """

    guild: _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    channel: _model.objects.Channel
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          - 
    """


class UpdateChannel(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.update_channel`.
    """

    guild: _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    channel: _model.objects.Channel
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          - 
    """


class DeleteChannel(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.delete_channel`.
    """

    guild: _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    channel: _model.objects.Channel
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          - 
    """


class CreateThread(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.create_thread`.
    """

    guild : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    thread: _model.objects.Channel
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          -
    """


class UpdateThread(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.update_thread`.
    """

    guild : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    thread: _model.objects.Channel
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          -
    """


class DeleteThread(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.delete_thread`.
    """

    guild : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    thread: _model.objects.Channel
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          -
    """


class SyncThreads(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.sync_threads`.
    """

    guild : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    threads: _model.types.Collection[_model.objects.Channel]
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`threads`
          -
    """


class UpdateThreadMember(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.update_thread_member`.
    """

    guild        : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    thread       : _model.objects.Channel
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`id`
          - :attr:`~.model.objects.Channel.id`
    """
    thread_member: _model.objects.ThreadMember
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          -
    """


class UpdateThreadMembers(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.update_thread_members`.
    """

    guild                 : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    thread                : _model.objects.Channel
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`id`
          - :attr:`~.model.objects.Channel.id`
    """
    created_thread_members: _model.types.Collection[_model.objects.ThreadMember]
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`added_members`
          - 
    """
    deleted_thread_members: _model.types.Collection[_model.objects.ThreadMember]
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`removed_member_ids`
          - :attr:`~.model.objects.Channel.id`
    """


class UpdateChannelPins(typing.NamedTuple):

    """
    Dispatched on :attr:`.enums.GatewayEvent.update_channel_pins`.
    """

    guild    : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    channel  : _model.objects.Channel
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`channel_id`
          - :attr:`~.model.objects.Channel.id`
    """
    timestamp: _model.types.ISO8601Timestamp
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`last_pin_timestamp`
          - 
    """


class CreateGuild(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.create_guild` when the guild is created.
    """

    guild: _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          -
    """


class AvailableGuild(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.create_guild` when the guild is available.
    """

    guild: _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          -
    """


class DeleteGuild(typing.NamedTuple):
  
    """
    Dispatched on :attr:`.enums.GatewayEvent.delete_guild` when the guild is deleted.
    """

    guild: _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          -
    """


class UnavailableGuild(typing.NamedTuple):
  
    """
    Dispatched on :attr:`.enums.GatewayEvent.` when the guild is unavailable.
    """

    guild: _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          -
    """


class UpdateGuild(typing.NamedTuple):
  
    """
    Dispatched on :attr:`.enums.GatewayEvent.update_guild`.

    Copiable with data before updating.
    """

    guild: _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          -
    """


class CreateGuildAuditLogEntry(typing.NamedTuple):
  
    """
    Dispatched on :attr:`.enums.GatewayEvent.create_guild_audit_log_entry`.

    Copiable with data before updating.
    """

    guild                : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    guild_audit_log_entry: _model.objects.AuditLogEntry
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          -
    """


class CreateGuildBan(typing.NamedTuple):
  
    """
    Dispatched on :attr:`.enums.GatewayEvent.create_guild_ban`.

    Copiable with data before updating.
    """

    guild: _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    user : _model.objects.User
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`user`
          -
    """


class DeleteGuildBan(typing.NamedTuple):
  
    """
    Dispatched on :attr:`.enums.GatewayEvent.delete_guild_ban`.

    Copiable with data before updating.
    """

    guild: _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    user : _model.objects.User
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`user`
          -
    """


class UpdateGuildEmojis(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.update_guild_emojis`.

    Copiable with data before updating.
    """

    guild: _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    emojis: _model.types.Collection[_model.objects.Emoji]
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`emojis`
          - 
    """


class UpdateGuildStickers(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.update_guild_stickers`.

    Copiable with data before updating.
    """

    guild: _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    emojis: _model.types.Collection[_model.objects.Emoji]
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`stickers`
          - 
    """


class UpdateGuildIntegrations(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.update_guild_integrations`.

    Copiable with data before updating.
    """

    guild: _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """


class CreateGuildMember(typing.NamedTuple):
  
    """
    Dispatched on :attr:`.enums.GatewayEvent.create_guild_member`.

    Copiable with data before updating.
    """

    guild       : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    guild_member: _model.objects.GuildMember
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          -
    """


class DeleteGuildMember(typing.NamedTuple):
  
    """
    Dispatched on :attr:`.enums.GatewayEvent.delete_guild_member`.

    Copiable with data before updating.
    """

    guild       : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    guild_member: _model.objects.GuildMember
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`user`
          -  :attr:`~.model.objects.GuildMember.user`
    """


class UpdateGuildMember(typing.NamedTuple):
  
    """
    Dispatched on :attr:`.enums.GatewayEvent.update_guild_member`.

    Copiable with data before updating.
    """

    guild       : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    guild_member: _model.objects.GuildMember
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          -
    """


class ReceiveGuildMembers(typing.NamedTuple):
  
    """
    Dispatched on :attr:`.enums.GatewayEvent.receive_guild_members`.

    Copiable with data before updating.
    """

    guild          : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    guild_members  : _model.types.Collection[_model.objects.GuildMember]
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`members`
          -
    """
    guild_presences: _model.types.Collection[_model.objects.Presence]
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`members`
          -
    """
    chunk_index    : int
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`chunk_index`
          -
    """
    chunk_indexes  : set[int]
    """
    The remaining indexes (receiving is complete when this is empty).

    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - internal
          -
    """


class CreateGuildRole(typing.NamedTuple):
  
    """
    Dispatched on :attr:`.enums.GatewayEvent.create_guild_role`.

    Copiable with data before updating.
    """

    guild     : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    guild_role: _model.objects.GuildMember
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`role`
          -
    """


class UpdateGuildRole(typing.NamedTuple):
  
    """
    Dispatched on :attr:`.enums.GatewayEvent.update_guild_role`.

    Copiable with data before updating.
    """

    guild     : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    guild_role: _model.objects.GuildMember
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`role`
          -
    """


class DeleteGuildRole(typing.NamedTuple):
  
    """
    Dispatched on :attr:`.enums.GatewayEvent.delete_guild_role`.

    Copiable with data before updating.
    """

    guild     : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    guild_role: _model.objects.GuildMember
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`role_id`
          - :attr:`~.model.objects.Role.id`
    """


class CreateGuildScheduledEvent(typing.NamedTuple):
  
    """
    Dispatched on :attr:`.enums.GatewayEvent.create_guild_scheduled_event`.

    Copiable with data before updating.
    """

    guild          : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    guild_scheduled_event: _model.objects.GuildMember
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          -
    """


class UpdateGuildScheduledEvent(typing.NamedTuple):
  
    """
    Dispatched on :attr:`.enums.GatewayEvent.update_guild_scheduled_event`.

    Copiable with data before updating.
    """

    guild          : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    guild_scheduled_event: _model.objects.GuildMember
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          -
    """


class DeleteGuildScheduledEvent(typing.NamedTuple):
  
    """
    Dispatched on :attr:`.enums.GatewayEvent.delete_guild_scheduled_event`.

    Copiable with data before updating.
    """

    guild          : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    guild_scheduled_event: _model.objects.GuildMember
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          -
    """


class CreateGuildScheduledEventUser(typing.NamedTuple):
  
    """
    Dispatched on :attr:`.enums.GatewayEvent.create_guild_scheduled_event_user`.

    Copiable with data before updating.
    """

    guild          : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    guild_scheduled_event: _model.objects.GuildMember
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_scheduled_event_id`
          - :attr:`~.model.objects.GuildScheduledEvent.id`
    """
    guild_member         : _model.objects.GuildMember
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`user_id`
          - :attr:`~.model.objects.GuildMember.user`\'s :attr:`~.model.objects.User.id`
    """


class DeleteGuildScheduledEventUser(typing.NamedTuple):
  
    """
    Dispatched on :attr:`.enums.GatewayEvent.delete_guild_scheduled_event_user`.

    Copiable with data before updating.
    """

    guild          : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    guild_scheduled_event: _model.objects.GuildMember
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_scheduled_event_id`
          - :attr:`~.model.objects.GuildScheduledEvent.id`
    """
    guild_member         : _model.objects.GuildMember
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`user_id`
          - :attr:`~.model.objects.GuildMember.user`\'s :attr:`~.model.objects.User.id`
    """


class CreateIntegration(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.create_integration`.
    """

    guild      : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    integration: _model.objects.Integration
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          - 
    """


class UpdateIntegration(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.update_integration`.
    """

    guild      : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    integration: _model.objects.Integration
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          - 
    """


class DeleteIntegration(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.delete_integration`.
    """

    guild      : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    integration: _model.objects.Integration
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          - 
    """


class CreateInvite(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.create_invite`.
    """

    guild  : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    channel: _model.objects.Channel
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`channel_id`
          - :attr:`~.model.objects.Channel.id`
    """
    invite : _model.objects.Invite
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          - 
    """


class DeleteInvite(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.delete_invite`.
    """

    guild  : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    channel: _model.objects.Channel
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`channel_id`
          - :attr:`~.model.objects.Channel.id`
    """
    invite : _model.objects.Invite
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`code`
          - :attr:`~.model.objects.Invite.code`
    """


class CreateMessage(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.create_message`.
    """

    guild  : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    channel: _model.objects.Channel
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`channel_id`
          - :attr:`~.model.objects.Channel.id`
        * - :code:`last_message_id`
          - :attr:`~.model.objects.Channel.last_message_id`
    """
    message: _model.objects.Message
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          -
    """


class UpdateMessage(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.update_message`.
    """

    guild  : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    channel: _model.objects.Channel
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`channel_id`
          - :attr:`~.model.objects.Channel.id`
        * - :code:`last_message_id`
          - :attr:`~.model.objects.Channel.last_message_id`
    """
    message: _model.objects.Message
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          -
    """


class DeleteMessage(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.delete_message`.
    """

    guild  : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    channel: _model.objects.Channel
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`channel_id`
          - :attr:`~.model.objects.Channel.id`
        * - :code:`last_message_id`
          - :attr:`~.model.objects.Channel.last_message_id`
    """
    message: _model.objects.Message
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`id`
          - :attr:`~.model.objects.Message.id`
    """


class DeleteMessages(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.delete_messages`.
    """

    guild   : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    channel : _model.objects.Channel
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`channel_id`
          - :attr:`~.model.objects.Channel.id`
        * - :code:`last_message_id`
          - :attr:`~.model.objects.Channel.last_message_id`
    """
    messages: _model.types.Collection[_model.objects.Message]
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`ids` (each)
          - :attr:`~.model.objects.Message.id`
    """


class CreateMessageReaction(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.create_message_reaction`.
    """

    guild       : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    guild_member: _model.objects.GuildMember
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`member`
          -
    """
    user        : _model.objects.User
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`user_id`
          - :attr:`~.model.objects.User.id`
    """
    channel     : _model.objects.Channel
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`channel_id`
          - :attr:`~.model.objects.Channel.id`
    """
    message     : _model.objects.Message
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`message_id`
          - :attr:`~.model.objects.Message.id`
    """
    emoji       : _model.objects.Emoji
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`emoji`
          -
    """


class DeleteMessageReaction(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.delete_message_reaction`.
    """

    guild       : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    guild_member: _model.objects.GuildMember
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`user_id`
          - :attr:`~.model.objects.GuildMember.user`\'s :attr:`~.model.objects.User.id`
    """
    user        : _model.objects.User
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`user_id`
          - :attr:`~.model.objects.User.id`
    """
    channel     : _model.objects.Channel
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`channel_id`
          - :attr:`~.model.objects.Channel.id`
    """
    message     : _model.objects.Message
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`message_id`
          - :attr:`~.model.objects.Message.id`
    """
    emoji       : _model.objects.Emoji
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`emoji`
          -
    """


class DeleteAllMessageReactions(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.delete_all_message_reactions`.
    """

    guild  : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    channel: _model.objects.Channel
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`channel_id`
          - :attr:`~.model.objects.Channel.id`
    """
    message: _model.objects.Message
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`message_id`
          - :attr:`~.model.objects.Message.id`
    """


class DeleteAllMessageEmojiReactions(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.delete_all_message_emoji_reactions`.
    """

    guild  : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    channel: _model.objects.Channel
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`channel_id`
          - :attr:`~.model.objects.Channel.id`
    """
    message: _model.objects.Message
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`message_id`
          - :attr:`~.model.objects.Message.id`
    """
    emoji  : _model.objects.Emoji
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`emoji`
          -
    """


class UpdatePresence(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.update_presence`.
    """

    guild   : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    presence: _model.objects.Presence
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          -
    """


class CreateTypingIndicator(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.create_typing_indicator`.
    """

    guild       : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    guild_member: _model.objects.GuildMember
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`member`
          - 
    """
    user        : _model.objects.GuildMember
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`user_id`
          - :attr:`~.model.objects.User.id`
    """
    channel     : _model.objects.Channel
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`channel_id`
          - :attr:`~.model.objects.Channel.id`
    """
    timestamp   : _model.types.Timestamp
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`timestamp`
          - 
    """


class UpdateSelfUser(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.update_self_user`.
    """

    user: _model.objects.User
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          - 
    """
    

class UpdateVoiceState(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.update_voice_state`.
    """

    guild      : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    voice_state: _model.objects.VoiceState
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          - 
    """
    voice      : _voice.client.Client | None
    """
    The self's voice client.
    """


class UpdateVoiceServer(typing.NamedTuple):

    """
    Dispatched on :attr:`.enums.GatewayEvent.update_voice_server`.
    """

    guild: _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    token: _model.types.String
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`token`
          -
    """
    uri: _model.types.String
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`uri`
          -
    """


class UpdateWebhooks(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.update_webhooks`.
    """

    guild: _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`guild_id`
          - :attr:`~.model.objects.Guild.id`
    """
    channel: _model.objects.Channel
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`channel_id`
          - :attr:`~.model.objects.Channel.id`
    """


class CreateInteraction(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.create_interaction`.
    """

    interaction: _model.objects.Interaction
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          -
    """


class CreateStageInstance(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.create_stage_instance`.
    """

    stage_instance: _model.objects.StageInstance
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          -
    """


class UpdateStageInstance(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.update_stage_instance`.
    """

    stage_instance: _model.objects.StageInstance
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          -
    """


class DeleteStageInstance(typing.NamedTuple):
    
    """
    Dispatched on :attr:`.enums.GatewayEvent.delete_stage_instance`.
    """

    stage_instance: _model.objects.StageInstance
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - body
          -
    """


class EnterVoice(typing.NamedTuple):

    """
    Dispatched on :attr:`.enums.VoiceEvent.enter`.
    """

    guild: _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :attr:`~.voice.client.Client.guild_id`
          - :attr:`~.model.objects.User.id`
    """
    user : _model.objects.GuildMember
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`user_id`
          - :attr:`~.model.objects.GuildMember.user`\'s :attr:`~.model.objects.User.id`
    """


class LeaveVoice(typing.NamedTuple):

    """
    Dispatched on :attr:`.enums.VoiceEvent.leave`.
    """

    guild: _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :attr:`~.voice.client.Client.guild_id`
          - :attr:`~.model.objects.User.id`
    """
    user : _model.objects.GuildMember
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`user_id`
          - :attr:`~.model.objects.GuildMember.user`\'s :attr:`~.model.objects.User.id`
    """


class Speak(typing.NamedTuple):

    """
    Dispatched on :attr:`.enums.VoiceEvent.speak`.
    """

    guild       : _model.objects.Guild
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :attr:`~.voice.client.Client.guild_id`
          - :attr:`~.model.objects.User.id`
    """
    user : _model.objects.GuildMember
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`user_id`
          - :attr:`~.model.objects.GuildMember.user`\'s :attr:`~.model.objects.User.id`
    """
    flags: _model.enums.SpeechFlags
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`speaking`
          -
    """
    ssrc : _model.types.Integer
    """
    .. list-table::
        :width: 450px
        :widths: 50 50
        :header-rows: 1

        * - Source
          - Target
        * - :code:`ssrc`
          -
    """



import vessel

missing = vessel.missing
"""
Returned by any attribute access that may have data, but does currently not.
"""

from . import enums
from . import types
from . import protocols
from . import images
from . import mentions
from . import objects
from . import responses


__all__ = (
    'missing', 'enums', 'types', 'protocols',
    'images', 'mentions', 'objects', 'responses'
)



import vessel

from . import types as _types
from . import enums as _enums


__all__ = (
    'user', 'channel', 'role', 'command', 'emoji', 'timestamp'
)


def _make(prefix, parts, delimiter = ':'):

    if any(part is vessel.missing for part in parts):
        return vessel.missing
    
    body = delimiter.join(map(str, parts))

    return f'<{prefix}{body}>'


def user(user_id: _types.Snowflake):

    parts = [user_id]

    return _make('@', parts)


def channel(channel_id: _types.Snowflake):

    parts = [channel_id]

    return _make('#', parts)


def role(role_id: _types.Snowflake):

    parts = [role_id]

    return _make('@&', parts)


def command(command_name: _types.String, command_id: _types.Snowflake):

    parts = [command_name, command_id]

    return _make('/', parts)


def emoji(emoji_name: _types.String, emoji_id: _types.Snowflake, animated: _types.Boolean = False):

    parts = ['a' if animated else '', emoji_name, emoji_id]

    return _make('', parts)


def timestamp(value: _types.Timestamp, style: _enums.TimestampStyle = None):

    parts = [value]

    if not style is None:
        parts.append(style)

    return _make('t', parts)



import typing
import struct
import asyncio

from . import errors as _errors


__all__ = ('Stream',)


class Stream:

    _prefix = b'OggS'
    _header_format = '<xBQIIIB'
    _header_size = struct.calcsize(_header_format)

    __slots__ = ('_read', '_queue', '_partial', '_task', '_event_fill')

    def __init__(self, read):

        self._read = read

        self._queue = asyncio.Queue()

        self._partial = b''

        self._task = None
        
        self._event_fill = asyncio.Event()

    async def _join(self):

        await self._event_fill.wait()

        await self._queue.join()

    def join(self) -> typing.Awaitable[None]:

        return self._join()

    async def _fill_next(self):

        prefix = await self._read(4)

        if not prefix == self._prefix:
           raise _errors.StreamPrefixMissing()

        header = await self._read(self._header_size)

        *_, segments_count = struct.unpack(self._header_format, header)

        segments_table = await self._read(segments_count)

        data_sizes = struct.unpack('B' * segments_count, segments_table)

        data = await self._read(sum(data_sizes))

        data = self._partial + data

        data_offset = 0
        packet_size = 0

        for data_size in data_sizes:
            packet_size += data_size
            if not data_size < 255:
                continue
            packet = data[data_offset:data_offset+packet_size]
            self._queue.put_nowait(packet)
            data_offset += packet_size
            packet_size = 0

        self._partial = data[data_offset:]

    async def _fill(self):

        while True:
            try:
                more = await self._fill_next()
            except _errors.StreamPrefixMissing:
                break
            self._event_fill.set()
            
    async def _get(self, wait):
        
        if wait:
            packet = await self._queue.get()
        else:
            try:
                packet = self._queue.get_nowait()
            except asyncio.QueueEmpty:
                raise _errors.StreamComplete()

        return packet
    
    def get(self, wait: bool) -> typing.Awaitable[None]:

        return self._get(wait)

    def _start(self):

        self._event_fill.clear()

        coro = self._fill()
        task = asyncio.create_task(coro)

        self._task = task

    def start(self):

        return self._start()

    def _stop(self):

        self._task.cancel()

    def stop(self):

        return self._stop()


import typing
import yarl
import vessel

from . import types as _types


__all__ = (
    'emoji', 'guild_icon', 'guild_splash', 'guild_discovery_splash', 
    'guild_banner', 'user_banner', 'default_user_avatar', 'user_avatar', 
    'guild_member_avatar', 'application_icon', 'application_cover', 
    'application_asset', 'achievement_icon', 'store_page_asset', 
    'sticker_pack_banner', 'team_icon', 'sticker', 'role_icon', 
    'guild_scheduled_event_cover', 'guild_member_banner'
)


_base_uri = yarl.URL('https://cdn.discordapp.com')


def _make(template, parts, extension = '.png', size = None):

    if any(part is vessel.missing for part in parts):
        return vessel.missing
    
    path = template.format(*parts)

    query = {}

    if not size is None:
        query['size'] = size

    return _base_uri.with_path(path + extension).with_query(query)


class _make_hint(typing.TypedDict):

    extension: typing.Optional[_types.String]
    size     : typing.Optional[_types.Integer]


def emoji(emoji_id: _types.Snowflake, /, **kwargs: typing.Unpack[_make_hint]):
    
    template = '/emojis/{0}'

    parts = [emoji_id]

    return _make(template, parts, **kwargs)


def guild_icon(guild_id: _types.Snowflake, guild_icon: _types.String, /, **kwargs: typing.Unpack[_make_hint]):
    
    template = '/icons/{0}/{1}'

    parts = [guild_id, guild_icon]

    return _make(template, parts, **kwargs)


def guild_splash(guild_id: _types.Snowflake, guild_splash: _types.String, /, **kwargs: typing.Unpack[_make_hint]):

    template = '/splashes/{0}/{1}'

    parts = [guild_id, guild_splash]

    return _make(template, parts, **kwargs)


def guild_discovery_splash(guild_id: _types.Snowflake, guild_discovery_splash: _types.String, /, **kwargs: typing.Unpack[_make_hint]):
    
    template = '/discovery-splashes/{0}/{1}'

    parts = [guild_id, guild_discovery_splash]
    
    return _make(template, parts, **kwargs)


def guild_banner(guild_id: _types.Snowflake, guild_banner: _types.String, /, **kwargs: typing.Unpack[_make_hint]):
    
    template = '/banners/{0}/{1}'

    parts = [guild_id, guild_banner]
    
    return _make(template, parts, **kwargs)


def user_banner(user_id: _types.Snowflake, user_banner: _types.String, /, **kwargs: typing.Unpack[_make_hint]):
    
    template = '/banners/{0}/{1}'

    parts = [user_id, user_banner]
    
    return _make(template, parts, **kwargs)


def default_user_avatar(user_discriminator: _types.Snowflake, /, **kwargs: typing.Unpack[_make_hint]):
    
    template = '/embed/avatars/{0}'

    parts = [user_discriminator % 5]
    
    return _make(template, parts, **kwargs)


def user_avatar(user_id: _types.Snowflake, user_avatar: _types.String, /, **kwargs: typing.Unpack[_make_hint]):
    
    template = '/avatars/{0}/{1}'

    parts = [user_id, user_avatar]
    
    return _make(template, parts, **kwargs)


def guild_member_avatar(guild_id: _types.Snowflake, user_id: _types.Snowflake, guild_member_avatar: _types.String, /, **kwargs: typing.Unpack[_make_hint]):
    
    template = '/guilds/{0}/users/{1}/avatars/{2}'

    parts = [guild_id, user_id, guild_member_avatar]
    
    return _make(template, parts, **kwargs)


def application_icon(application_id: _types.Snowflake, icon: _types.String, /, **kwargs: typing.Unpack[_make_hint]):
    
    template = '/app-icons/{0}/{1}'

    parts = [application_id, icon]
    
    return _make(template, parts, **kwargs)


def application_cover(application_id: _types.Snowflake, cover_image: _types.String, /, **kwargs: typing.Unpack[_make_hint]):
    
    template = '/app-icons/{0}/{1}'

    parts = [application_id, cover_image]
    
    return _make(template, parts, **kwargs)


def application_asset(application_id: _types.Snowflake, asset_id: _types.String, /, **kwargs: typing.Unpack[_make_hint]):
    
    template = '/app-assets/{0}/{1}'

    parts = [application_id, asset_id]
    
    return _make(template, parts, **kwargs)


def achievement_icon(application_id: _types.Snowflake, achievement_id: _types.Snowflake, icon_hash: _types.String, /, **kwargs: typing.Unpack[_make_hint]):
    
    template = '/app-assets/{0}/achievements/{1}/icons/{2}'

    parts = [application_id, achievement_id, icon_hash]
    
    return _make(template, parts, **kwargs)


def store_page_asset(application_id: _types.Snowflake, asset_id: _types.String, /, **kwargs: typing.Unpack[_make_hint]):
    
    # NOTE: spec does not link "asset_id"
    template = '/app-assets/{0}/store/{1}'

    parts = [application_id, asset_id]

    return _make(template, parts, **kwargs)


def sticker_pack_banner(sticker_pack_banner_asset_id: _types.Snowflake, /, **kwargs: typing.Unpack[_make_hint]):
    
    template = '/app-assets/710982414301790216/store/{0}'

    parts = [sticker_pack_banner_asset_id]
    
    return _make(template, parts, **kwargs)


def team_icon(team_id: _types.Snowflake, team_icon: _types.String, /, **kwargs: typing.Unpack[_make_hint]):
    
    template = '/team-icons/{0}/{1}'

    parts = [team_id, team_icon]
    
    return _make(template, parts, **kwargs)


def sticker(sticker_id: _types.Snowflake, /, **kwargs: typing.Unpack[_make_hint]):
    
    template = '/stickers/{0}'

    parts = [sticker_id]
    
    return _make(template, parts, **kwargs)


def role_icon(role_id: _types.Snowflake, role_icon: _types.String, /, **kwargs: typing.Unpack[_make_hint]):
    
    template = '/role-icons/{0}/{1}'

    parts = [role_id, role_icon]
    
    return _make(template, parts, **kwargs)


def guild_scheduled_event_cover(scheduled_event_id: _types.Snowflake, scheduled_event_cover: _types.String, /, **kwargs: typing.Unpack[_make_hint]):

    template = '/guild-events/{0}/{1}'

    parts = [scheduled_event_id, scheduled_event_cover]

    return _make(template, parts, **kwargs)


def guild_member_banner(guild_id: _types.Snowflake, user_id: _types.Snowflake, member_banner: _types.String, /, **kwargs: typing.Unpack[_make_hint]):

    template = '/guilds/{0}/users/{1}/banners/{2}'

    parts = [guild_id, user_id, member_banner]

    return _make(template, parts, **kwargs)


import asyncio
import aiocord

async def start(client):
    await client.start()
    await client.ready()

async def main(client):
    voice = await client.start_voice('506536485156739358', '543273600667152384')
    audio = aiocord.voice.audio.Audio(source = './song.mp3')
    await voice.player.start(audio)

async def stop(client):
    await client.stop()

loop = asyncio.get_event_loop()

client = aiocord.client.Client(token = ...)

loop.run_until_complete(start(client))

try:
    loop.run_until_complete(main(client))
except KeyboardInterrupt:
    pass
finally:
    loop.run_until_complete(stop(client))


import typing
import vessel
import datetime

from . import enums as _enums


__all__ = (
    'String', 'Boolean', 'Integer', 'Decimal', 'List', 'Dict',
    'Timestamp', 'ISO8601Timestamp', 'Snowflake'
)


Object = vessel.Object


Collection = vessel.Collection


String = str


Boolean = bool


Integer = int


Decimal = float


List = list


Dict = dict


class Timestamp(Integer):

    """
    A `Unix Timestamp <https://www.unixtimestamp.com>`_.
    """

    __slots__ = ()

    @property
    def datetime(self) -> datetime.datetime:

        """
        The datetime object representation.
        """

        return datetime.datetime.fromtimestamp(self)
    
    def mention(self, style: _enums.TimestampStyle = None):

        """
        Get the mention.

        :param style:
            The visual representation type.
        """

        from . import _mentions

        return _mentions.timestamp(self, style = style)


class ISO8601Timestamp(String):

    """
    |dsrc|
    :ddoc:`ISO8601 Date/Time </reference#iso8601-datetime>`
    """

    __slots__ = ()

    @property
    def datetime(self) -> datetime.datetime:

        """
        The datetime object representation.
        """

        return datetime.datetime.fromisoformat(self)
    

class Snowflake(String):

    """
    |dsrc|
    :ddoc:`Snowflakes </reference#snowflakes>`
    """

    __slots__ = ()

    @property
    def timestamp(self, epoch = 1420070400000) -> Timestamp:

        """
        The internal timestamp.
        """

        return Timestamp((int(self) >> 22) + epoch)


import setuptools

with open('README.rst', encoding = 'utf-8') as file:
    readme = file.read()

author = 'Exahilosys'
project = 'aiocord'

url = 'https://github.com/{0}/{1}'.format(author, project)

setuptools.setup(
    name = project,
    python_requires = '>=3.11',
    use_scm_version = True,
    setup_requires = [
        'setuptools_scm'
    ],
    url = url,
    packages = setuptools.find_packages(),
    license = 'MIT',
    long_description = readme,
    description = 'A modern API wrapper for Discord.',
    requires = [
        'yarl    (>= 1.9.1, < 2.0.0)',
        'aiohttp (>= 3.8.4, < 4.0.0)',
        'vessel  (>= 4.2.0, < 5.0.0)',
        'pynacl  (>= 1.5.0, < 2.0.0)'
    ],
    extras_require = {
        'docs': [
            'sphinx',
            'sphinx-paramlinks',
            'sphinx-autodoc-typehints'
        ]
    },
    entry_points = {
        'console_scripts': [
            f'{project}={project}.vendor:main'
        ]
    }
)


import importlib.metadata


__all__ = ()


def clean_query(store):

    for key in tuple(store):
        value = store[key]
        if value is None:
            del store[key]; continue
        if isinstance(value, bool):
            value = int(value)
        if isinstance(value, int):
            value = str(value)
        store[key] = value


def clean_headers(store):

    for key in tuple(store):
        value = store[key]
        if value is None:
            del store[key]; continue
        store[key] = str(value)


def get_user_agent():

    name = __name__.split('.', 1)[0]

    try:
        metadata = importlib.metadata.metadata(name)
    except importlib.metadata.PackageNotFoundError:
        return None

    url = metadata['Home-Page']
    version = metadata['Version']

    user_agent = f'DiscordBot ({url} {version})'

    return user_agent


import aiocord

commands = [
    aiocord.model.protocols.ApplicationCommand(
        name = 'blep',
        type = aiocord.model.enums.ApplicationCommandType.chat_input,
        description = 'blep someone',
        options = [
            aiocord.model.protocols.ApplicationCommandOption(
                name = 'user',
                description = 'the user to blep',
                type = aiocord.model.enums.ApplicationCommandOptionType.user
            )
        ]
    )
]


import typing
import sys
import os
import asyncio
import subprocess

from . import stream as _stream


__all__ = ('Audio',)


class Audio:

    """
    Manages a FFMPEG subprocess for converting and encoding audio.
    
    :param |dsrc|
        The source of the file, or :code:`'-'` if data is going to be fed later.
    :param executable:
        The path to the FFMPEG executable.
        
    All other parameters should remain as-is according to discord's specification.
    """

    def __init__(self, 
                 source        : str | typing.Literal['-']= '-',
                 executable    : str = 'ffmpeg',
                 codec         : str | None = 'opus',
                 sample_rate   : int = 48000,
                 channels      : int = 2, 
                 bit_depth     : int = 16, 
                 frame_duration: int = 0.02):
        
        self._source = source
        self._executable = executable

        self._sample_rate = sample_rate
        self._channels = channels
        self._bit_depth = bit_depth
        self._frame_duration = frame_duration

        self._codec  = codec
        self._stream = _stream.Stream(self._read)

        self._latent = bytearray()

        self._subprocess = None

    @property
    def sample_rate(self):

        return self._sample_rate
    
    @property
    def channels(self):

        return self._channels
    
    @property
    def frame_duration(self):

        return self._frame_duration

    @property
    def _sample_bytes_count(self):

        return int(self._bit_depth / 8 * self._channels)
    
    @property
    def sample_bytes_count(self):

        return self._sample_bytes_count
    
    @property
    def _frame_samples_count(self):

        return int(self._frame_duration * self._sample_rate)
    
    @property
    def frame_samples_count(self):

        return self._frame_samples_count
    
    @property
    def _frame_bytes_count(self):

        return self._frame_samples_count * self._sample_bytes_count
    
    @property
    def frame_bytes_count(self):

        return self._frame_bytes_count
    
    @property
    def _frame_samples_count(self):

        return int(self._frame_duration * self._sample_rate)
    
    @property
    def frame_samples_count(self):

        return self._frame_samples_count
    
    @property
    def _frame_bytes_count(self):

        return self._frame_samples_count * self._audio.sample_bytes_count
    
    @property
    def frame_bytes_count(self):

        return self._frame_bytes_count
    
    @property
    def join(self):

        return self._stream.join
    
    def _feed(self, data):

        if self._subprocess is None:
            self._latent.extend(data)
        else:
            self._subprocess.stdin.write(data)

    def feed(self, data: bytes) -> None:

        """
        Feed data if :code:`'-'` has been used for the source.

        :param data:
            The data to write to the subprocess.
        """

        return self._feed(data)
    
    async def _read(self, size):

        data = await self._subprocess.stdout.read(size)
        
        return data
    
    async def _get(self):

        wait = self._subprocess.returncode is None

        packet = await self._stream.get(wait)

        return packet
    
    def get(self) -> typing.Awaitable[bytes]:

        """
        Get a single audio packet.
        """

        return self._get()
    
    def _get_ffmpeg_arguments_codec_copy(self):

        arguments = [
            '-c:a', 'copy',
        ]

        return arguments

    def _get_ffmpeg_arguments_codec_opus(self):

        arguments = [
            '-c:a', 'libopus',
            '-b:a', '128K',
            '-frame_duration', f'{int(self._frame_duration * 1000)}',
            '-packet_loss', '0.15',
            '-fec', '1',
            '-cutoff', '20000'
        ]

        return arguments

    def _get_ffmpeg_arguments(self):

        arguments = [
            '-loglevel', 'warning',
            '-i', self._source, 
            '-vn', 
            '-ar', f'{self._sample_rate}', 
            '-ac', f'{self._channels}',
        ]

        if codec := self._codec:
            generate = getattr(self, f'_get_ffmpeg_arguments_codec_{codec}')
            arguments.extend(generate())

        arguments.extend(
            [
                '-f', 'opus',
                'pipe:1'
            ]
        )

        return arguments

    async def _start(self):

        arguments = self._get_ffmpeg_arguments()

        direct = self._source == '-'

        stdin = subprocess.PIPE if direct else None

        self._subprocess = await asyncio.create_subprocess_exec(
            self._executable, *arguments, 
            stdin = stdin, stdout = subprocess.PIPE
        )

        if direct:
            self._subprocess.stdin.write(self._latent)
            self._latent.clear()

        self._stream.start()

    def start(self) -> typing.Awaitable[None]:

        """
        Create the subprocess for feeding and reading data.
        """

        return self._start()
    
    async def _stop(self):

        self._stream.stop()

        try:
            self._subprocess.kill()
        except ProcessLookupError:
            pass
        else:
            await self._subprocess.wait()

    def stop(self) -> typing.Awaitable[None]:

        """
        Stop the reading data and close the subprocess.
        """

        return self._stop()
    
